"use strict";
/**
 * key-value-compress.ts
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.compressKVS = void 0;
const _compress = require("./_compress");
const _transform_1 = require("./_transform");
const _util_1 = require("./_util");
/**
 * Key-Value storage interceptor to deflate, split, concatenate and inflate content
 */
function compressKVS(options) {
    let { chunkNS, chunkSize, compress, digest, encoding } = options || {};
    let { inlineSize, metaNS, metaStorage, storage } = options || {};
    if (!metaStorage)
        metaStorage = storage;
    if (metaNS)
        metaStorage = _transform_1.namespaceKVS(metaStorage, metaNS);
    const metaKSV = _transform_1.jsonKVS(metaStorage);
    if (chunkNS)
        storage = _transform_1.namespaceKVS(storage, chunkNS);
    if (encoding)
        storage = _util_1.base64KVS(storage, encoding);
    let compressor = _compress[compress || "deflate" /* compress */];
    if (inlineSize == null)
        inlineSize = 1024 /* inlineSize */;
    if (!chunkSize)
        chunkSize = 491520 /* chunkSize */;
    if (!digest)
        digest = "sha1" /* digest */;
    return { get, set };
    async function set(key, value) {
        const meta = { v: 1 /* version */, chunks: [] };
        let raw;
        if (Buffer.isBuffer(value)) {
            raw = value;
            meta.type = "Buffer";
        }
        else {
            if ("string" === typeof value) {
                raw = Buffer.from(value);
                meta.type = "string";
            }
            else {
                raw = Buffer.from(JSON.stringify(value));
                meta.type = "JSON";
            }
        }
        const compressed = await compressor.compress(raw);
        const chunks = _util_1.splitBuffer(compressed, chunkSize);
        for (let chunk of chunks) {
            const hash = _util_1.digestBuffer(digest, chunk);
            if (chunk.length <= inlineSize) {
                const inline = meta.inline || (meta.inline = {});
                inline[hash] = chunk.toString("base64");
            }
            else {
                await storage.set(hash, chunk);
            }
            meta.chunks.push(hash);
        }
        await metaKSV.set(key, meta);
    }
    async function get(key) {
        var _a;
        const meta = await metaKSV.get(key);
        if ((meta === null || meta === void 0 ? void 0 : meta.v) !== 1 /* version */)
            return;
        if (!((_a = meta.chunks) === null || _a === void 0 ? void 0 : _a.length))
            return;
        const chunks = [];
        for (const hash of meta.chunks) {
            const base64 = meta.inline && meta.inline[hash];
            let chunk;
            if (base64) {
                chunk = Buffer.from(base64, "base64");
            }
            else {
                chunk = await storage.get(hash);
            }
            if (!chunk)
                return;
            const check = _util_1.digestBuffer(digest, chunk);
            if (check !== hash)
                return;
            chunks.push(chunk);
        }
        const joined = _util_1.concatBuffer(chunks);
        const buffer = await compressor.decompress(joined);
        if (!buffer)
            return;
        let value;
        if (meta.type === "Buffer") {
            value = buffer;
        }
        else if (meta.type === "string") {
            value = buffer.toString();
        }
        else {
            value = JSON.parse(buffer);
        }
        return value;
    }
}
exports.compressKVS = compressKVS;
